/** Interface for NSComparisonPredicate for GNUstep
   Copyright (C) 2005 Free Software Foundation, Inc.

   Written by:  Quentin Mathe <qmatheclub-internet.fr>
   Date: May 2005
   
   This file is part of the GNUstep Base Library.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.

   AutogsdocSource: NSComparisonPredicate.m
   */ 

#ifndef __NSComparisonPredicate_h_GNUSTEP_BASE_INCLUDE
#define __NSComparisonPredicate_h_GNUSTEP_BASE_INCLUDE

#if OS_API_VERSION(100400,GS_API_LATEST) && GS_API_VERSION(010200,GS_API_LATEST)

// FIXME: Update GS_API_VERSION when new base release is on the way.

#include <Foundation/NSPredicate.h>

@class NSExpression;

/*
 * Constants
 */

typedef enum _NSComparisonPredicateModifier
{
  NSAllPredicateModifier,
  NSAnyPredicateModifier,
  NSDirectPredicateModifier
} NSComparisonPredicateModifier;

typedef enum _NSComparisonPredicateOption
{
  NSCaseInsensitivePredicateOption,
  NSDiacriticInsensitivePredicateOption
} NSComparisonPredicateOption;

typedef enum _NSPredicateOperatorType
{
  NSBeginsWithPredicateOperatorType,
  NSCustomSelectorPredicateOperatorType,
  NSEndsWithPredicateOperatorType,
  NSEqualToPredicateOperatorType,
  NSGreaterThanPredicateOperatorType,
  NSGreaterThanOrEqualToPredicateOperatorType,
  NSInPredicateOperatorType,
  NSLessThanOrEqualToPredicateOperatorType,
  NSLikePredicateOperatorType,
  NSMatchesPredicateOperatorType,
  NSNotEqualToPredicateOperatorType
} NSPredicateOperatorType;


@interface NSComparisonPredicate : NSPredicate
{
  NSExpression *_lhs;
  NSExpression *_rhs;
  SEL _customSelector;
  NSComparisonPredicateModifier _predicateModifier;
  NSPredicateOperatorType _operatorType;
  unsigned _options;
}

/*
 * Factory methods
 */

+ (NSPredicate *) predicateWithLeftExpression: (NSExpression *)lhs 
                                        rightExpression: (NSExpression *)rhs
                                         customSelector: (SEL)selector;
 
+ (NSPredicate *) predicateWithLeftExpression: (NSExpression *)lhs 
                              rightExpression: (NSExpression *)rhs 
                                     modifier:
                                       (NSComparisonPredicateModifier)modifier 
                                         type: (NSPredicateOperatorType)type 
                                      options: (unsigned)options;

/*
 * Other methods
 */  

- (NSComparisonPredicateModifier) comparisonPredicateModifier;
- (SEL) customSelector;

/*
 * Init methods
 */

- (NSPredicate *) initWithLeftExpression: (NSExpression *)lhs 
                         rightExpression: (NSExpression *)rhs 
                          customSelector: (SEL)selector;
                         
- (id) initWithLeftExpression: (NSExpression *)lhs 
              rightExpression: (NSExpression *)rhs 
                     modifier: (NSComparisonPredicateModifier)modifier 
                         type: (NSPredicateOperatorType)type 
                      options: (unsigned)options;

/*
 * Other methods
 */

- (NSExpression *) leftExpression;
- (unsigned) options;
- (NSPredicateOperatorType) predicateOperatorType;
- (NSExpression *) rightExpression;

@end

#endif	/* API_VERSION */
#endif	/* __NSComparisonPredicate_h_GNUSTEP_BASE_INCLUDE*/
