/** Interface for NSExpression for GNUstep
   Copyright (C) 2005 Free Software Foundation, Inc.

   Written by:  Quentin Mathe <qmatheclub-internet.fr>
   Date: May 2005
   
   This file is part of the GNUstep Base Library.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.

   AutogsdocSource: NSExpression.m
   */ 

#ifndef __NSExpression_h_GNUSTEP_BASE_INCLUDE
#define __NSExpression_h_GNUSTEP_BASE_INCLUDE

#if OS_API_VERSION(100400,GS_API_LATEST) && GS_API_VERSION(010200,GS_API_LATEST)

// FIXME: Update GS_API_VERSION when new base release is on the way.

#include <Foundation/NSObject.h>

@class NSArray, NSDictionary, NSMutableArray, NSMutableDictionary, NSString;

/*
 * Constants
 */
 
typedef enum _NSExpressionType
{
  NSConstantValueExpressionType,
  NSEvaluatedObjectExpressionType,
  NSFunctionExpressionType,
  NSKeyPathExpressionType,
  NSVariableExpressionType
} NSExpressionType;
 

@interface NSExpression : NSObject
{
  NSExpressionType _exprType;
  id _exprOperand; /* Evaluated object */
  id _exprConstantValue;
  id _exprValue;
  NSString *_functionName;
  NSMutableArray *_functionArguments;
  BOOL _exprBoolValue; /* For optimization */
}

/*
 * Factory methods
 */
 
+ (NSExpression *)expressionForConstantValue: (id)obj;
+ (NSExpression *)expressionForEvaluatedObject;
+ (NSExpression *)expressionForFunction: (NSString *)name 
                              arguments: (NSArray *)parameters;
+ (NSExpression *)expressionForKeyPath: (NSString *)keyPath;
+ (NSExpression *)expressionForVariable:(NSString *)string;

/*
 * Other methods
 */
 
- (NSArray *)arguments;
- (id)constantValue;
- (NSExpressionType)expressionType;
- (id)expressionValueWithObject: (id)object 
                        context: (NSMutableDictionary *)context;
- (NSString *)function;

/*
 * Init methods
 */
 
- (id)initWithExpressionType: (NSExpressionType)type;

/*
 * Other methods
 */
 
- (NSString *)keyPath;
- (NSExpression *)operand;
- (NSString *)variable;

@end

#endif	/* API_VERSION */
#endif	/* __NSExpression_h_GNUSTEP_BASE_INCLUDE*/
