/** Interface for NSPredicate for GNUstep
   Copyright (C) 2005 Free Software Foundation, Inc.

   Written by:  Quentin Mathe <qmatheclub-internet.fr>
   Date: May 2005
   
   This file is part of the GNUstep Base Library.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.

   AutogsdocSource: NSPredicate.m
   */ 

#ifndef __NSPredicate_h_GNUSTEP_BASE_INCLUDE
#define __NSPredicate_h_GNUSTEP_BASE_INCLUDE

#if OS_API_VERSION(100400,GS_API_LATEST) && GS_API_VERSION(010200,GS_API_LATEST)

// FIXME: Update GS_API_VERSION when new base release is on the way.

#include <Foundation/NSObject.h>

@class NSArray, NSDictionary, NSString, NSExpression;

@interface NSPredicate : NSObject
{
  NSExpression *_rootExpr;
}

/*
 * Factory methods
 */
 
+ (NSPredicate *) predicateWithFormat: (NSString *)format, ...;
+ (NSPredicate *) predicateWithFormat: (NSString *)predicateFormat 
                        argumentArray: (NSArray *)arguments;
+ (NSPredicate *) predicateWithFormat: (NSString *)format 
                            arguments: (va_list)argList;
+ (NSPredicate *) predicateWithValue: (BOOL)value;

/*
 * Other methods
 */
 
- (BOOL) evaluateWithObject: (id)object;
- (NSString *) predicateFormat;
- (NSPredicate *) predicateWithSubstitutionVariables: (NSDictionary *)variables;

@end

#endif	/* API_VERSION */
#endif	/* __NSPredicate_h_GNUSTEP_BASE_INCLUDE*/
