#include "ObjectTesting.h"
#include <Foundation/NSAutoreleasePool.h>
#include <Foundation/NSPredicate.h>
#include <Foundation/NSCompoundPredicate.h>
#include <Foundation/NSComparisonPredicate.h>
#include <Foundation/NSExpression.h>


int main()
{
  NSPredicate *predicate, *compoundPredicate, *comparisonPredicate;
  NSExpression *expr1, *expr2;
  NSMutableArray *testObjects;

  CREATE_AUTORELEASE_POOL(arp);

  testObjects = [NSMutableArray array];

  /* NSExpression related tests */

  expr1 = [NSExpression expressionForConstantValue: @"blabla"];
  pass(expr1 != nil, "We can create a constant expression (YES)");
  [testObjects addObject: expr1];

  expr1 = [NSExpression expressionForEvaluatedObject];
  pass(expr1 != nil, "We can create an evaluated object expression (YES)");
  [testObjects addObject: expr1];

  expr1 = [NSExpression expressionForFunction: @"count" 
    arguments: [NSArray arrayWithObjects: 
    [NSExpression expressionForConstantValue: @"untitled1"], 
    [NSExpression expressionForConstantValue: @"untitled2"], nil]];
  pass(expr1 != nil, "We can create a function expression (YES)");
  [testObjects addObject: expr1];

  expr1 = [NSExpression expressionForKeyPath: @"className"];
  pass(expr1 != nil, "We can create a key path expression (YES)");
  [testObjects addObject: expr1];

  expr2 = [NSExpression expressionForVariable: @"boundValue"];
  pass(expr2 != nil, "We can create a variable expression (YES)");
  [testObjects addObject: expr2];

  test_NSObject(@"NSExpression", testObjects);
  //test_NSCoding(testObjects);
  //test_NSCopying(testObjects);

  [testObjects removeAllObjects];

  /* NSPredicate and subclasses related tests */

  predicate = [NSPredicate predicateWithValue: YES];
  pass(predicate != nil, "We can create a predicate (YES)");
  [testObjects addObject: predicate];

  comparisonPredicate = 
   [NSComparisonPredicate predicateWithLeftExpression: expr1 
                            rightExpression: expr2 
                             customSelector: @selector(isEqual:)];
  pass(comparisonPredicate != nil, 
    "We can create a comparison predicate with custom selector (YES)");
  [testObjects addObject: comparisonPredicate];

  comparisonPredicate = [NSComparisonPredicate 
    predicateWithLeftExpression: expr1 
                rightExpression: expr2 
                       modifier: NSDirectPredicateModifier
                           type: NSEqualToPredicateOperatorType
                        options: NSCaseInsensitivePredicateOption];
  pass(comparisonPredicate != nil, 
    "We can create a comparison predicate with built-in options (YES)");
  [testObjects addObject: comparisonPredicate];

  compoundPredicate = 
    [NSCompoundPredicate andPredicateWithSubpredicates: testObjects];
  pass(compoundPredicate != nil, "We can create a 'and' coumpound predicate \
    (YES)");
  [testObjects addObject: compoundPredicate];

  compoundPredicate = 
    [NSCompoundPredicate orPredicateWithSubpredicates: testObjects];
  pass(compoundPredicate != nil, "We can create a 'or' coumpound predicate \
    (YES)");
  [testObjects addObject: compoundPredicate];

  compoundPredicate = 
    [NSCompoundPredicate notPredicateWithSubpredicate: predicate];
  pass(compoundPredicate != nil, "We can create a 'not' coumpound predicate \
    (YES)");
  [testObjects addObject: compoundPredicate];

  test_NSObject(@"NSPredicate, NSCompoundPredicate, NSComparisonPredicate",
    testObjects);
  //test_NSCoding(testObjects);
  //test_NSCopying(testObjects);

  DESTROY(arp);
  return 0;
}
